#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"

#define MAZE_WIDTH      30
#define MAZE_HEIGHT     20
#define TILE_SIZE       20
#define WALL_WIDTH      2

#define SCREEN_WIDTH    (MAZE_WIDTH  * TILE_SIZE + WALL_WIDTH)
#define SCREEN_HEIGHT   (MAZE_HEIGHT * TILE_SIZE + WALL_WIDTH)

struct STile
{
    bool    topIsActive;
    bool    leftIsActive;

    Color   topColor;
    Color   leftColor;
};

STile   maze[MAZE_WIDTH + 1][MAZE_HEIGHT + 1];

void drawWall(int x, int y, bool isHorizontal, Color c)
{
    int sx = x * TILE_SIZE;
    int sy = y * TILE_SIZE;

    if (isHorizontal == true)
    {
        int ex = sx + TILE_SIZE - 1;
        int ey = sy + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
    else
    {
        int ex = sx + WALL_WIDTH - 1;
        int ey = sy + TILE_SIZE + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Tree Maze 2", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0));

    // init the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            maze[x][y].topIsActive  = false;
            maze[x][y].leftIsActive = false;
        }

    // init left and right borders
    for (int y = 0; y < MAZE_HEIGHT; y++)
    {
        maze[0][y].leftIsActive = true;
        maze[0][y].leftColor = Color(255, 255, 255);

        maze[MAZE_WIDTH][y].leftIsActive = true;
        maze[MAZE_WIDTH][y].leftColor = Color(255, 255, 255);
    }

    // init top and bottom borders
    for (int x = 0; x < MAZE_WIDTH; x++)
    {
        maze[x][0].topIsActive = true;
        maze[x][0].topColor = Color(255, 255, 255);

        maze[x][MAZE_HEIGHT].topColor = Color(255, 255, 255);
        maze[x][MAZE_HEIGHT].topIsActive = true;
    }

    // draw the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            if (maze[x][y].leftIsActive == true)
                drawWall(x, y, false, maze[x][y].leftColor);

            if (maze[x][y].topIsActive == true)
                drawWall(x, y, true, maze[x][y].topColor);
        }

    // wait till the end
    while (sys.isQuitRequested() == false)
    {
        sys.processEvents();
        gfx.render();
        sys.wait(50);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
